package com.ruoyi.file.service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.file.mapper.FileSequenceMapper;
import com.ruoyi.file.mapper.FilesMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@Service
public class FilesServiceImpl extends ServiceImpl<FilesMapper, Files> implements FilesService {
    @Value("${file.path}")
    private String filesUploadPath;//获取文件路径

    @Value("${file.domain}")
    private String filesUploadDomain;//文件域名

    @Value("${file.prefix}")
    private String filesPrefix;//文件前缀

    @Autowired
    private FileSequenceMapper fileSequenceMapper;


    @Override
    public FileUploadResponse upload(MultipartFile file) {

        // 文件夹路径名称
        String originalFilename = file.getOriginalFilename();

        //文件大小
        double size = file.getSize() / (1024.0);

        // 获取当前日期时间
        LocalDateTime now = LocalDateTime.now();
        String timestamp = now.format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
        int monthIncrement = getMonthIncrement(now.getYear(), now.getMonthValue()); // 获取每月递增值

        //文件类型
        String type = originalFilename.substring(originalFilename.lastIndexOf(".") + 1).toLowerCase();

        // 按照年月创建文件夹
        String yearMonth = now.format(DateTimeFormatter.ofPattern("yyyyMM"));
        // 新文件名格式：时间戳+每月递增值+文件类型
        String fileUuid = timestamp + "A" + String.format("%04d", monthIncrement) + "." + type;

        // 将相对路径转化为绝对路径
//        String destPath = new File(filesUploadPath + "/" + yearMonth).getAbsolutePath();//绝对路径
        String destPath = filesUploadPath + "/" + yearMonth;

        // 新的文件地址，绝对路径+新的文件名称
        File uploadFile = new File(destPath + "/" + fileUuid);

        // 判断配置的文件目录是否存在，若不存在则创建一个新的文件目录
        File parentFile = uploadFile.getParentFile();
        if (!parentFile.exists()) {
            parentFile.mkdirs();
        }

        try {
            // 存储文件到本地磁盘
            file.transferTo(uploadFile);

            // 设置文件url
            String url = filesUploadDomain + filesPrefix + "/" + yearMonth + "/" + fileUuid;

            // 将文件存储到数据库
            Files saveFile = new Files();
            saveFile.setFilesName(originalFilename);
            saveFile.setName(fileUuid);
            saveFile.setPath(destPath);
            saveFile.setType(type);
            saveFile.setSize(size); // （单位：KB）
            saveFile.setUrl(url);
            saveFile.setEnable(true);
            saveFile.setCreateTime(LocalDateTime.now());
            saveFile.setUpdateTime(null);
            // 保存操作
            save(saveFile);

            // 返回 FileUploadResponse 对象
            return new FileUploadResponse(fileUuid, url);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    private int getMonthIncrement(int year, int month) {
        // 查询当前月份的递增值
        FileSequence increment = fileSequenceMapper.getByYearAndMonth(year, month);
        if (increment == null) {
            // 如果没有记录，则初始化为1
            increment = new FileSequence();
            increment.setYear(year);
            increment.setMonth(month);
            increment.setValue(1);
            fileSequenceMapper.insert(increment);
        } else {
            // 更新递增值
            increment.setValue(increment.getValue() + 1);
            fileSequenceMapper.updateById(increment);
        }
        return increment.getValue();
    }

    //将文件以流的形式一次性读取到内存，通过响应输出流输出到前端
    @Override
    public void download(String filesUUID, HttpServletResponse response) {
        try {
            //根据文件的唯一标识码获取文件
            File uploadFile = new File(filesUploadPath + filesUUID);

            //读取文件的字节流
            FileInputStream fileInputStream = new FileInputStream(uploadFile);
            //将文件写入输入流
            InputStream inputStream = new BufferedInputStream(fileInputStream);

            byte[] buffer = new byte[inputStream.available()];
            inputStream.read(buffer);
            inputStream.close();


            //attachment表示以附件方式下载 inline表示在线打开 "Content-Disposition: inline; filename=文件名.png"
            //filename表示文件的默认名称，因为网络传输只支持URL编码的相关支付，因此需要将文件名URL编码后进行传输,前端收到后需要反编码才能获取到真正的名称
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filesUUID, "UTF-8"));
            response.setContentType("application/octet-stream");

            //设置输出流的格式
            ServletOutputStream os = response.getOutputStream();
            os.write(buffer);

            //关闭
            fileInputStream.close();
            os.flush();
            os.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}